# reads an openbox themerc and spits out a similar theme for visiblity.

# represents an X rgb file
class RgbFile
	attr_reader :colours
	def initialize(rgb_path = "")
		@colours = {} # maps key->array of r, g, b
		if rgb_path.length
			parse rgb_path
		end
	end
	def parse(rgb_path)
		rgb_file = File.new(rgb_path)
		rgb_file.each_line do |line|
			red, green, blue, name = line.split
			@colours[name] = [red.to_i, green.to_i, blue.to_i]
		end
	end
end

class Colour
	# this file is used to parse things like 'grey60'; you may need to adjust
	# this path to suit your system.
	RgbPath = '/usr/share/X11/rgb.txt'
	ColourNames = RgbFile.new(RgbPath).colours
	def initialize(colour_string)
		@red = @green = @blue = 0 # values from 0..255
		if colour_string =~ /^rgb:/
			colour_string = colour_string[4..-1] # skip the rgb:
			red, green, blue = colour_string.split('/')
			# not really as tricky as it looks
			hex_to_fraction = lambda do |hex|
				hex.to_i(16).to_f / (2.0 ** (hex.length.to_f * 4.0) - 1.0)
			end
			@red = 255 * hex_to_fraction.call(red)
			@green = 255 * hex_to_fraction.call(green)
			@blue = 255 * hex_to_fraction.call(blue)
		elsif (pos = colour_string =~ /[a-fA-f0-9]{6}$/)
			colour_string = colour_string[pos..-1] # skip whatever leads
			@red = colour_string[0..1].to_i(16)
			@green = colour_string[2..3].to_i(16)
			@blue = colour_string[4..5].to_i(16)
		else
			@red, @green, @blue = ColourNames[colour_string]
		end
	end
	def to_hex
		red = @red.to_i.to_s(16)
		green = @green.to_i.to_s(16)
		blue = @blue.to_i.to_s(16)
		red, green, blue = [red, green, blue].map do |colour|
			if colour.length == 1
				colour = "0" + colour
			end
			colour
		end
		red + green + blue
	end
end

class OpenboxTheme
	attr_reader :values
	def initialize(filename = "")
		@values = Hash.new
		if filename.length
			parse(filename)
		end
	end
	def parse(filename)
		file = File.new(filename)
		file.each_line do |line|
			stripped = line.strip
			next if stripped.empty?
			next if stripped[0] == '!'
			key, value = stripped.split(/\s*:\s*/, 2)
			next if (key == nil || value == nil)
			@values[key] = value
		end
	end
end

class VisibilityTheme
	# a hash that maps visibility key -> openbox key
	OpenboxThemeKeys = {
		'spacing' => 'padding.width',
		'border_width' => 'border.width',
		'desktop_separation' => 'padding.width',
		'bg_colour' => ['window.inactive.label.bg.color',
				'window.inactive.title.bg.color'],
		'border_colour' => 'border.color',
		'active_text_colour' => 'window.active.label.text.color',
		'inactive_text_colour' => 'window.inactive.label.text.color',
		'font' => 'window.active.label.text.font',
		'inactive_bg_colour' => ['window.inactive.label.bg.color',
				'window.inactive.title.bg.color'],
		'active_bg_colour' => ['window.active.label.bg.color',
				'window.active.title.bg.color'],
		'tooltip_padding' => 'padding.width',
		'tooltip_bg_colour' => 'menu.title.bg.color',
		'tooltip_border_colour' => 'border.color',
		'tooltip_text_colour' => 'menu.title.text.color',
		'tooltip_font' => 'menu.title.text.font',
	}
	attr_reader :values
	def initialize(openbox_theme)
		@values = {}
		OpenboxThemeKeys.each do |visibility_key, openbox_key|
			if openbox_key.class == Array
				value = ''
				openbox_key.each do |key|
					value = openbox_theme.values[key]
					next if value == nil
					next if value.empty?
					next if value == 'parentrelative'
					if key =~ /color$/
						@values[visibility_key] = Colour.new(value).to_hex
					else
						@values[visibility_key] = value
					end
					break
				end
			else
				if openbox_key =~ /color$/
					@values[visibility_key] = Colour.new(openbox_theme.values[openbox_key]).to_hex
				else
					@values[visibility_key] = openbox_theme.values[openbox_key]
				end
			end
		end
	end
end

exit if ARGV.empty?

themefile = ARGV[0]
theme = VisibilityTheme.new(OpenboxTheme.new(themefile))
theme.values.each do |key, value|
	next if value == nil
	next if value.empty?
	if value[0] == 35 # is this wrong?
		value = value[1..-1]
	end
	puts key + ' ' + value
end
