#include "Colour.h"
#include <sstream>
#include <stdio.h>

static const unsigned colour_length = 6;

namespace Visibility {

static void parse_colour_24(CARD32 colour, double &red, double &green,
	double &blue) {

	red = double((colour >> 16) & 0xff) / 255.0;
	green = double((colour >> 8) & 0xff) / 255.0;
	blue = double(colour & 0xff) / 255.0;
}

static void parse_colour_16(CARD32 colour, double &red, double &green,
	double &blue) {

	red = double((colour >> 11) & 0x1f) / 31.0;
	green = double((colour >> 5) & 0x3f) / 63.0;
	blue = double(colour & 0x1f) / 31.0;
}

Colour::Colour(std::string colour) {
	unsigned length = 0;
	if ((length = colour.length()) > colour_length) {
		// drop the first extra chars
		colour.erase(0, length - colour_length);
	}

	CARD32 pixel = 0;
	std::stringstream stream(colour, std::ios::in);

	stream >> std::hex >> pixel;
	parse_colour_24(pixel, red, green, blue);
}

Colour::Colour(CARD32 colour, unsigned char bits_per_pixel) {
	switch(bits_per_pixel) {
		case 16:
			parse_colour_16(colour, red, green, blue);
			break;
		default:
			parse_colour_24(colour, red, green, blue);
			break;
	}
}

CARD32 Colour::pixel_value(unsigned char bits_per_pixel) {
	CARD32 pixel = 0;
	unsigned char red = 0, green = 0, blue = 0;
	switch(bits_per_pixel) {
		case 24:
			red = (unsigned char)(this->red * 255.0);
			green = (unsigned char)(this->green * 255.0);
			blue = (unsigned char)(this->blue * 255.0);
			pixel = (0xff << 24) | (red << 16) | (green << 8) | blue;
			break;
		case 16:
			red = (unsigned char)(this->red * 31.0);
			green = (unsigned char)(this->green * 63.0);
			blue = (unsigned char)(this->blue * 31.0);
			pixel = (red << 11) | (green << 5) | blue;
			break;
	}

	return pixel;
}

}
