#ifndef DESKTOP_HEADER_INCLUDED
#define DESKTOP_HEADER_INCLUDED

#include "DesktopName.h"
#include "WindowIcon.h"
#include <map>
#include <string>
#include <vector>

#include <sigc++/sigc++.h>
#include <memory>

namespace Visibility {

	class Desktop {
	public:
		Desktop(::Window parent, CARD32 number);
		~Desktop();

		// emitted when the width is modified
		sigc::signal<void> signal_width_changed;

		inline unsigned get_width(void) const {
			return width;
		}

		std::tr1::shared_ptr<Window> main;

		// signal handlers
		void window_closed_callback(::Window window);
		void window_opened_callback(::Window window);

		static void update_desktop_names(void);

	protected:
		static std::vector<std::string> desktop_names;

		void reconfigure(void); // reload config

		// resizes the window and pixmaps
		void resize(void);
		unsigned calculate_width(void);
		void initialize(void);

		// puts the desktop name and icons in place
		void position_internals(void);

		std::tr1::shared_ptr<DesktopName> name;
		std::string name_string;

		unsigned width;
		CARD32 number; // which desktop am i?

		bool is_active;

		// this will change the window background pixmap, and change the text
		// colour
		void desktop_changed_callback(void);
		void desktop_name_changed_callback(void);
		void desktop_name_clicked_callback(::Window window);

		std::map< ::Window, std::tr1::shared_ptr<WindowIcon> > window_icons;
		std::map< ::Window, std::vector<sigc::connection> > window_signals;
		sigc::connection signal_new_window_connection;
		sigc::connection signal_desktop_changed_connection;
		sigc::connection signal_config_changed_connection;
		sigc::connection signal_desktop_name_changed_connection;
	};

}

#endif
